#ifndef COMPLEXE_HPP
#define COMPLEXE_HPP

#include <cmath>
#include <iostream>
using namespace std;

template <class T> class complexe {
  T r;
  T i;
public:
  complexe (){
    r = T();
    i = T();

  }
  complexe (T reel , T img){
    r = reel;  i = img;
  }
  complexe (const complexe & x){
    r = x.r;  i = x.i;
  }
  ~complexe (){
  }
  T reel(){
    return r;
  }
  T img(){
    return i;
  }
  complexe modify_reel(T reel){
    r = reel;
    return *this;
  };
  complexe modify_img(T img){
    i = img;
    return *this;
  };
  double  module();
  complexe conjugue();

  bool reel_pure();
  bool img_pure();
  complexe operator + (complexe);
  complexe operator - ( complexe);
  complexe operator * ( complexe);
  complexe operator / ( complexe);
  complexe & operator = (const complexe &);
  bool operator < (complexe);
  bool operator > (complexe);
  bool operator == (complexe);
  bool operator != (complexe);
  template <class U>
  friend ostream & operator<<(ostream &, const complexe<U> &);
  template <class U>
  friend istream & operator>>(istream &, complexe<U> &);
  template <class U>
  friend complexe<U> operator *(int, const complexe<U> &);
  int testu_i1();
  int testu_i2();
  int testu_d1();
  int testu_d2();
  int all_testsu();
};

template <class T> bool complexe<T>::reel_pure(){
  bool res = false;
  if( i == 0.0) { res = true; }
  return res;
}

template <class T> bool complexe<T>::img_pure(){
  bool res = false;
  if( r == 0.0) { res = true; }
  return res;
}

template <class T> double complexe<T>::module(){
  double res=0;
  return sqrt(r*r+i*i);
}

template <class T> complexe<T> complexe<T>::conjugue(){
  complexe<T> res;
  res.r = r;
  res.i = -i;
  return res;
} 

template <class T> complexe<T> complexe<T>:: operator + (complexe<T> x){
  complexe<T> res;
  res.r = r+x.r;
  res.i = i+x.i;
  return res;
};

template <class T> complexe<T> complexe<T>:: operator * (complexe<T> x){
  complexe<T> res;
  res.r = r*x.r-i*x.i;
  res.i = r*x.i+i*x.r;
  return res;
};

template <class T> complexe<T> complexe<T>:: operator - (complexe<T> x){
  complexe<T> res;
  res.r = r-x.r;
  res.i = i-x.i;
  return res;
};

template <class T> complexe<T> complexe<T> :: operator / (complexe<T> x){
  complexe<T> res;
  double mod;
  mod = this->module();
  res.r = (r*x.r+i*x.i)/(mod*mod);
  res.i = (i*x.r-x.i*r)/(mod*mod);
  return res;
};

template <class T> complexe<T> & complexe<T>:: operator = (const complexe<T> & x){
  if(&x != this){
    r = x.r;
    i = x.i;
  }
  return *this;
};

template <class T> bool complexe<T>:: operator < (complexe<T> x){
  bool res = false;
  if(this->module() < x.module() ) { res = true;}
  return res;
};

template <class T> bool complexe<T>:: operator > (complexe<T> x){
  bool res = false;
  if(this->module() > x.module()) { res = true;}
  return res;
};

template <class T> bool complexe<T>:: operator == (complexe<T> x){
  bool res = false;
  if(this->module() == x.module()) { res = true;}
  return res;
};

template <class T> ostream & operator<<(ostream & out, const complexe<T> & x){
  out<<x.r<<"+i "<<x.i;
  return out;
}

template <class T> istream & operator>>(istream & in,complexe<T> & x){
  cout<<" partie reel:"<<endl;
  in>>x.r;
  cout<<" partie img:"<<endl;
  in>>x.i;
  return in;
}

template <class T> complexe<T> operator *(int a, const complexe<T> & x){
  complexe<T> res;
  T rr, ii;
  rr = a*x.r;
  ii = a*x.i;
  res.r = rr;
  res.i = ii;
  return res;
}


template <class T> int complexe<T>:: testu_d1(){
  complexe<double> x1(1.0,1.0);
  complexe<double> x2(1.0,-2.0);
  double mod=0.0;
  
  *this = x1;
  *this = *this + x2 ; // *this = 2 -i
  x1 = x1.conjugue(); // x1  = 1 - i
  *this = *this * x1 ; // *this = 1 -3i
  *this = *this / x1; // *this = (1-3i)/(1-i) = (4- 2i)/10
  if(this->reel() > 0.399 && this->img() < -0.199) { return 0; }
  else { return 1; }
}

template <class T> int complexe<T>:: testu_i1(){
  complexe<int> x1(1,1);
  complexe<int> x2(1,-2);
  int mod=0;
  
  *this = x1;
  *this = *this + x2 ; // *this = 2 -i
  x1 = x1.conjugue(); // x1  = 1 - i
  *this = *this * x1 ; // *this = 1 -3i
  // cout<<x1<<endl;
  if(this->reel() == 1 && this->img() == -3) { return 0; }
  else { return 1; }
}

template <class T> int complexe<T>:: testu_d2(){
  complexe<double> x1(1.0,1.0);
  bool a=false,b=false,c=false,d=false;
  
   *this = x1;
   a= this->reel_pure();
   b= this->img_pure();
   this->modify_reel(0.0);
   c = this->img_pure();
   this->modify_reel(1.0);
   this->modify_img(0.0);
   d = this->reel_pure();

   if( a+b+c+d == 2) { return 0; }
   else { return 1; }
  return 0;
}

template <class T> int complexe<T>:: testu_i2(){
  complexe<int> x1(1,1);
  bool a=false,b=false,c=false,d=false;
  
  *this = x1;
  a= this->reel_pure();
  b= this->img_pure();
  this->modify_reel(0);
  c = this->img_pure();
  this->modify_reel(1);
  this->modify_img(0);
  d = this->reel_pure();

  if( a+b+c+d == 2) { return 0; }
  else { return 1; }
  return 0;
}

template <class T> int complexe<T>:: all_testsu(){
   int c=0,t=0;
   complexe<int> xi;
   complexe<double> xd;
   c=xi.testu_i1(); t = t +c;
   c=xi.testu_i2(); t = t +c;
   c=xd.testu_d1(); t = t +c;
   c=xd.testu_d2(); t = t +c;
   cout<<"Tests unitaires réussis, classe complexe: "<<4-t<<"/4"<<endl;
   return 0;
}

#endif
